% CubeModel_simulationResults.m
%
% Generates the simulation results for Balancing Cube model (Sec. III-B), 
% which are presented in Fig. 3.
%
% --
% Supplementary material to
%
% S. Trimpe, R. D'Andrea, "Event-Based State Estimation with Variance-Based 
% Triggering", IEEE Transactions on Automatic Control, Special Issue on 
% Control of Cyber-Physical Systems.
%
% Copyright:
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% strimpe(at)tuebingen.mpg.de
% March 2014
%

% Revision history
% [Nov 12, ST]   first version
% [Dec 13, ST]   adapt equation numbers etc. to revised vrsion of the paper
% [10.03.14, ST] adapt to revised version of the paper

clear;

disp(' ');
disp('## Section III-B, Balancing Cube model.');
disp('## Results from simulating (21).');
disp(' ');



%% Configuration
% Uncomment to use latex interpreter.
%set(0,'defaulttextinterpreter','latex');    % set text interpreter

% plot transmit thresholds
flag_plotDelta = true;



%% Load Kalman filter parameters.
% See corresponding script for details.
EventBasedEstimatorParameters;



%% Simulation parameters.
% Total number of simulation steps.
K = 1800;



%% Simulation: Iterate the variance equation.
% Iteration of the variance iteration (21):
%
%   P(k+1) = H(P(k)).
%
% For the implementation of H see the file map_H.m.

% Variables to hold the simulation results.
P = zeros(n,n,K+1);
gamma = nan(M,K);

% Initialize with solution of the DARE.
P(:,:,1) = P_bar;
for k=1:K
    [P(:,:,k+1),gamma(:,k)] = map_H(P(:,:,k),A,C,Q,R,P_bar,delta);
end;


% %% Display the transmit sequence
% % Compute transmit sequence for 1 period such that 1<=k<=50.
% kkPeriod = 1001:1050;   % kkPeriod=1 corresponds to k=1 in that period
% 
% disp('# Transmit instants:');
% for j=1:M
%     transmitInstants = find(gamma(j,kkPeriod));
%     disp(['Sensor ',num2str(j),': ',num2str(transmitInstants)]);
% end;


%% Plot the results.

% Figure 1: plot of data corresponding to Fig. 3 in paper:  plot the
% elements P_{11}, P_{12}, P_{77}, P_{88}.
%
fig1 = figure(1); 

kk = 1000:1150;     % time range tp plot, 3 periods

% P_{11}: variance of module 1's angle
subplot(4,1,1);
plot(kk,squeeze(P(1,1,kk)),'k.-');
grid;
if flag_plotDelta
    % Active threshold for P(1,1) is delta(1)+P_bar(1,1).
    thrsh = delta(1) + P_bar(1,1);
    hold on;
    plot([kk(1),kk(end)],[thrsh, thrsh],'k--','Linewidth',0.8);
end;
set(gca,'ylim',[0, 53]);
set(gca,'ytick',0:10:50);
ylabel(['$P_{11}(k)$']);

% P_{12}: covariance module angles 1 and 2
subplot(4,1,2);
plot(kk,squeeze(P(1,2,kk)),'k.-');
grid;
% No active threshold on P(1,2).
set(gca,'ylim',[0, 0.42]);
%set(gca,'ytick',0:10:50);
ylabel(['$P_{12}(k)$']);

% P_{77}: variance of cube angle
subplot(4,1,3);
plot(kk,squeeze(P(7,7,kk)),'k.-');
grid;
% No active threshold on P(7,7).
set(gca,'ylim',[0.73, 0.87]);
ylabel(['$P_{77}(k)$']);

% P_{88}: variance of cube angular velocity
subplot(4,1,4);
plot(kk,squeeze(P(8,8,kk)),'k.-');
grid;
if flag_plotDelta
    % Active threshold for P(8,8) is delta(2)+P_bar(8,8).
    thrsh = delta(2) + P_bar(8,8);
    hold on;
    plot([kk(1),kk(end)],[thrsh, thrsh],'k--','Linewidth',0.8);
end;
set(gca,'ylim',[0.7, 6.3]);
set(gca,'ytick',1:1:6);
ylabel(['$P_{88}(k)$']);
xlabel('Iteration $k$');

disp(' ');
disp('# Simulation results:');
disp(['The plot in Matlab Figure ',num2str(fig1),' is the result shown in Fig. 3 of the paper ']);
disp('"Event-Based State Estimation with Variance-Based Triggering".');
disp(' ');


% % Figure 2: all elements of the estimator variance \check{P}(k) 
% % (for completeness, figure is very crowded).
% figure(2);
% for i=1:n
%     for j=1:n
%         subplot(n,n,(i-1)*n+j);
%         plot(squeeze(P(i,j,:)));
%         grid;
%     end;
% end;
